@@ -1341,7 +1341,8 @@ elf_hppa_relocate_section (output_bfd, info, input_bfd, input_section,
 		relocation = 0;
 	    }
 	  /* Allow undefined symbols in shared libraries.  */
-          else if (info->shared && !info->no_undefined)
+          else if (info->shared && !info->no_undefined
+		   && ELF_ST_VISIBILITY (h->other) == STV_DEFAULT)
 	    {
 	      if (info->symbolic)
 		(*info->callbacks->undefined_symbol)
@@ -1520,7 +1521,10 @@ elf_hppa_final_link_relocate (rel, input_bfd, output_bfd,
 		  + input_section->output_section->vma);
 
 	/* Adjust for any field selectors.  */
-	value = hppa_field_adjust (value, -8 + addend, e_fsel);
+	if (r_type == R_PARISC_PCREL17R)
+	  value = hppa_field_adjust (value, -8 + addend, e_rsel);
+	else
+	  value = hppa_field_adjust (value, -8 + addend, e_fsel);
 
 	/* All branches are implicitly shifted by 2 places.  */
 	value >>= 2;
@@ -1680,8 +1684,9 @@ elf_hppa_final_link_relocate (rel, input_bfd, output_bfd,
     case R_PARISC_DIR16DF:
       {
 	/* All DIR relocations are basically the same at this point,
-	   except that we need different field selectors for the 21bit
-	   version vs the 14bit versions.  */
+	   except that branch offsets need to be divided by four, and
+	   we need different field selectors.  Note that we don't
+	   redirect absolute calls to local stubs.  */
 
 	if (r_type == R_PARISC_DIR21L)
 	  value = hppa_field_adjust (value, addend, e_lrsel);
@@ -1693,6 +1698,12 @@ elf_hppa_final_link_relocate (rel, input_bfd, output_bfd,
 	else
 	  value = hppa_field_adjust (value, addend, e_rrsel);
 
+	if (r_type == R_PARISC_DIR17R || r_type == R_PARISC_DIR17F)
+	  {
+	    /* All branches are implicitly shifted by 2 places.  */
+	    value >>= 2;
+	  }
+
 	insn = elf_hppa_relocate_insn (insn, value, r_type);
 	break;
       }
@@ -1965,7 +1976,7 @@ elf_hppa_relocate_insn (insn, sym_value, r_type)
        the "B" instruction.  */
     case R_PARISC_PCREL22F:
     case R_PARISC_PCREL22C:
-      return re_assemble_22 (insn, sym_value);
+      return re_assemble_22 (insn & ~ 0x3ff1ffd, sym_value);
 
     /* This is any 17bit branch.  In PA2.0 syntax it also corresponds to
        the "B" instruction as well as BE.  */
@@ -1974,7 +1985,7 @@ elf_hppa_relocate_insn (insn, sym_value, r_type)
     case R_PARISC_DIR17R:
     case R_PARISC_PCREL17C:
     case R_PARISC_PCREL17R:
-      return re_assemble_17 (insn, sym_value);
+      return re_assemble_17 (insn & ~ 0x1f1ffd, sym_value);
 
     /* ADDIL or LDIL instructions.  */
     case R_PARISC_DLTREL21L:
@@ -1985,7 +1996,7 @@ elf_hppa_relocate_insn (insn, sym_value, r_type)
     case R_PARISC_DPREL21L:
     case R_PARISC_PLTOFF21L:
     case R_PARISC_DIR21L:
-      return re_assemble_21 (insn, sym_value);
+      return re_assemble_21 (insn & ~ 0x1fffff, sym_value);
 
     /* LDO and integer loads/stores with 14bit displacements.  */
     case R_PARISC_DLTREL14R:
