@@ -141,6 +141,7 @@ DESCRIPTION
 .  bfd_target_aout_flavour,
 .  bfd_target_coff_flavour,
 .  bfd_target_ecoff_flavour,
+.  bfd_target_xcoff_flavour,
 .  bfd_target_elf_flavour,
 .  bfd_target_ieee_flavour,
 .  bfd_target_nlm_flavour,
@@ -505,7 +506,12 @@ extern const bfd_target arm_epoc_pei_little_vec;
 extern const bfd_target arm_epoc_pei_big_vec;
 extern const bfd_target b_out_vec_big_host;
 extern const bfd_target b_out_vec_little_host;
+extern const bfd_target bfd_efi_app_ia32_vec;
+extern const bfd_target bfd_efi_app_ia64_vec;
 extern const bfd_target bfd_elf64_alpha_vec;
+extern const bfd_target bfd_elf64_hppa_vec;
+extern const bfd_target bfd_elf64_ia64_little_vec;
+extern const bfd_target bfd_elf64_ia64_big_vec;
 extern const bfd_target bfd_elf32_avr_vec;
 extern const bfd_target bfd_elf32_bigarc_vec;
 extern const bfd_target bfd_elf32_bigarm_vec;
@@ -527,6 +533,8 @@ extern const bfd_target bfd_elf32_little_generic_vec;
 extern const bfd_target bfd_elf32_littlemips_vec;
 extern const bfd_target bfd_elf64_littlemips_vec;
 extern const bfd_target bfd_elf32_m32r_vec;
+extern const bfd_target bfd_elf32_m68hc11_vec;
+extern const bfd_target bfd_elf32_m68hc12_vec;
 extern const bfd_target bfd_elf32_m68k_vec;
 extern const bfd_target bfd_elf32_m88k_vec;
 extern const bfd_target bfd_elf32_mn10200_vec;
@@ -609,6 +617,7 @@ extern const bfd_target ppcboot_vec;
 extern const bfd_target riscix_vec;
 extern const bfd_target pmac_xcoff_vec;
 extern const bfd_target rs6000coff_vec;
+extern const bfd_target rs6000coff64_vec;
 extern const bfd_target shcoff_vec;
 extern const bfd_target shlcoff_vec;
 extern const bfd_target shcoff_small_vec;
@@ -625,6 +634,12 @@ extern const bfd_target sunos_big_vec;
 extern const bfd_target tekhex_vec;
 extern const bfd_target tic30_aout_vec;
 extern const bfd_target tic30_coff_vec;
+extern const bfd_target tic54x_coff0_vec;
+extern const bfd_target tic54x_coff0_beh_vec;
+extern const bfd_target tic54x_coff1_vec;
+extern const bfd_target tic54x_coff1_beh_vec;
+extern const bfd_target tic54x_coff2_vec;
+extern const bfd_target tic54x_coff2_beh_vec;
 extern const bfd_target tic80coff_vec;
 extern const bfd_target vaxnetbsd_vec;
 extern const bfd_target versados_vec;
@@ -683,6 +698,11 @@ const bfd_target * const bfd_target_vector[] = {
 	&b_out_vec_big_host,
 	&b_out_vec_little_host,
 
+	&bfd_efi_app_ia32_vec,
+#ifdef BFD64
+	&bfd_efi_app_ia64_vec,
+#endif
+
 	/* This, and other vectors, may not be used in any *.mt configuration.
 	   But that does not mean they are unnecessary.  If configured with
 	   --enable-targets=all, objdump or gdb should be able to examine
@@ -690,6 +710,9 @@ const bfd_target * const bfd_target_vector[] = {
 	&bfd_elf32_big_generic_vec,
 #ifdef BFD64
 	&bfd_elf64_alpha_vec,
+	&bfd_elf64_hppa_vec,
+	&bfd_elf64_ia64_little_vec,
+	&bfd_elf64_ia64_big_vec,
 #endif
 	&bfd_elf32_avr_vec,
 	&bfd_elf32_bigarc_vec,
@@ -701,9 +724,7 @@ const bfd_target * const bfd_target_vector[] = {
 #endif
 	&bfd_elf32_d10v_vec,
 	&bfd_elf32_d30v_vec,
-#if 0
 	&bfd_elf32_hppa_vec,
-#endif
 	&bfd_elf32_i370_vec,
 	&bfd_elf32_i386_vec,
 	&bfd_elf32_i860_vec,
@@ -719,6 +740,8 @@ const bfd_target * const bfd_target_vector[] = {
 	&bfd_elf32_m32r_vec,
 	&bfd_elf32_mn10200_vec,
 	&bfd_elf32_mn10300_vec,
+	&bfd_elf32_m68hc11_vec,
+	&bfd_elf32_m68hc12_vec,
 	&bfd_elf32_m68k_vec,
 	&bfd_elf32_m88k_vec,
 	&bfd_elf32_sparc_vec,
@@ -848,6 +871,9 @@ const bfd_target * const bfd_target_vector[] = {
 	&pmac_xcoff_vec,
 #endif
 	&rs6000coff_vec,
+#ifdef BFD64
+	&rs6000coff64_vec,
+#endif
 	&ppcboot_vec,
 	&shcoff_vec,
 	&shlcoff_vec,
@@ -862,6 +888,12 @@ const bfd_target * const bfd_target_vector[] = {
 	&aout0_big_vec,
 	&tic30_aout_vec,
 	&tic30_coff_vec,
+	&tic54x_coff0_vec,
+	&tic54x_coff0_beh_vec,
+	&tic54x_coff1_vec,
+	&tic54x_coff1_beh_vec,
+	&tic54x_coff2_vec,
+	&tic54x_coff2_beh_vec,
 	&tic80coff_vec,
 	&vaxnetbsd_vec,
 	&versados_vec,
