@@ -1,5 +1,5 @@
 /* addr2line.c -- convert addresses to line number and function name
-   Copyright 1997, 1998 Free Software Foundation, Inc.
+   Copyright 1997, 98, 99, 2000 Free Software Foundation, Inc.
    Contributed by Ulrich Lauther <Ulrich.Lauther@zfe.siemens.de>
 
    This file is part of GNU Binutils.
@@ -43,27 +43,26 @@ static boolean with_functions;	/* -f, show function names.  */
 static boolean do_demangle;	/* -C, demangle names.  */
 static boolean base_names;	/* -s, strip directory names.  */
 
-/* --lang=xxx */
-static int demangle_lang = DMGL_ANSI | DMGL_PARAMS;
-
 static int naddr;		/* Number of addresses to process.  */
 static char **addr;		/* Hex addresses to process.  */
 
 static asymbol **syms;		/* Symbol table.  */
 
 /* 150 isn't special; it's just an arbitrary non-ASCII char value.  */
-#define OPTION_LANG (150)
+#define OPTION_DEMANGLER	(150)
+#define OPTION_STYLE		(OPTION_DEMANGLER + 1)
 
 static struct option long_options[] =
 {
   {"basenames", no_argument, NULL, 's'},
   {"demangle", no_argument, NULL, 'C'},
+  {"demangler", required_argument, NULL, OPTION_DEMANGLER},
   {"exe", required_argument, NULL, 'e'},
   {"functions", no_argument, NULL, 'f'},
+  {"style", required_argument, NULL, OPTION_STYLE},
   {"target", required_argument, NULL, 'b'},
   {"help", no_argument, NULL, 'H'},
   {"version", no_argument, NULL, 'V'},
-  {"lang", required_argument, NULL, OPTION_LANG},
   {0, no_argument, 0, 0}
 };
 
@@ -73,8 +72,6 @@ static void find_address_in_section PARAMS ((bfd *, asection *, PTR));
 static void translate_addresses PARAMS ((bfd *));
 static void process_file PARAMS ((const char *, const char *));
 
-extern void binutils_bug_report PARAMS((FILE *));
-
 /* Print a usage message to STREAM and exit with STATUS.  */
 
 static void
@@ -82,14 +79,18 @@ usage (stream, status)
      FILE *stream;
      int status;
 {
-  fprintf (stream, "\
+  fprintf (stream, _("\
 Usage: %s [-CfsHV] [-b bfdname] [--target=bfdname]\n\
-       [-e executable] [--exe=executable] [--demangle] [--lang=language]\n\
-       [--basenames] [--functions] [addr addr ...]\n",
+       [-e executable] [--exe=executable]\n\
+       [--demangle] [--demangler=dso:function]\n"),
 	   program_name);
+  fprintf (stream, _("\
+       [--style=%s]\n\
+       [--basenames] [--functions] [addr addr ...]\n"),
+       get_demangler_list ());
   list_supported_targets (program_name, stream);
   if (status == 0)
-    binutils_bug_report (stream);
+    fprintf (stream, _("Report bugs to %s\n"), REPORT_BUGS_TO);
   exit (status);
 }
 
@@ -132,9 +133,10 @@ static void
 find_address_in_section (abfd, section, data)
      bfd *abfd;
      asection *section;
-     PTR data;
+     PTR data ATTRIBUTE_UNUSED;
 {
   bfd_vma vma;
+  bfd_size_type size;
 
   if (found)
     return;
@@ -146,6 +148,10 @@ find_address_in_section (abfd, section, data)
   if (pc < vma)
     return;
 
+  size = bfd_get_section_size_before_reloc (section);
+  if (pc >= vma + size)
+    return;
+
   found = bfd_find_nearest_line (abfd, section, syms, pc - vma,
 				 &filename, &functionname, &line);
 }
@@ -190,7 +196,7 @@ translate_addresses (abfd)
 	{
 	  if (with_functions)
 	    {
-	      if (*functionname == '\0')
+	      if (functionname == NULL || *functionname == '\0')
 		printf ("??\n");
 	      else if (! do_demangle)
 		printf ("%s\n", functionname);
@@ -198,7 +204,7 @@ translate_addresses (abfd)
 		{
 		  char *res;
 
-		  res = cplus_demangle (functionname, demangle_lang);
+		  res = demangle_symbol (functionname);
 		  if (res == NULL)
 		    printf ("%s\n", functionname);
 		  else
@@ -209,7 +215,7 @@ translate_addresses (abfd)
 		}
 	    }
 
-	  if (base_names)
+	  if (base_names && filename != NULL)
 	    {
 	      char *h;
 
@@ -218,7 +224,7 @@ translate_addresses (abfd)
 		filename = h + 1;
 	    }
 
-	  printf ("%s:%u\n", filename, line);
+	  printf ("%s:%u\n", filename ? filename : "??", line);
 	}
 
       /* fflush() is essential for using this command as a server
@@ -244,7 +250,7 @@ process_file (filename, target)
     bfd_fatal (filename);
 
   if (bfd_check_format (abfd, bfd_archive))
-    fatal ("%s: can not get addresses from archive", filename);
+    fatal (_("%s: can not get addresses from archive"), filename);
 
   if (! bfd_check_format_matches (abfd, bfd_object, &matching))
     {
@@ -278,6 +284,13 @@ main (argc, argv)
   char *filename;
   char *target;
   int c;
+  const char *demangler = NULL, *style = NULL;
+
+#if defined (HAVE_SETLOCALE) && defined (HAVE_LC_MESSAGES)
+  setlocale (LC_MESSAGES, "");
+#endif
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
   program_name = *argv;
   xmalloc_set_program_name (program_name);
@@ -315,9 +328,11 @@ main (argc, argv)
 	case 'H':
 	  usage (stdout, 0);
 	  break;
-	case OPTION_LANG:
-	  if (strcasecmp (optarg, "gnat") == 0)
-	    demangle_lang = DMGL_GNAT;
+	case OPTION_DEMANGLER:
+	  demangler = optarg;
+	  break;
+	case OPTION_STYLE:
+	  style = optarg;
 	  break;
 	default:
 	  usage (stderr, 1);
@@ -325,6 +340,8 @@ main (argc, argv)
 	}
     }
 
+  init_demangler (style, NULL, demangler);
+
   if (filename == NULL)
     filename = "a.out";
 
