@@ -256,11 +256,14 @@ static bfd *lineno_cache_bfd;
 static bfd *lineno_cache_rel_bfd;
 
 #define OPTION_TARGET 200
+#define OPTION_DEMANGLER (OPTION_TARGET + 1)
+#define OPTION_STYLE (OPTION_DEMANGLER + 1)
 
 static struct option long_options[] =
 {
   {"debug-syms", no_argument, &print_debug_syms, 1},
   {"demangle", no_argument, &do_demangle, 1},
+  {"demangler", required_argument, NULL, OPTION_DEMANGLER},
   {"dynamic", no_argument, &dynamic, 1},
   {"extern-only", no_argument, &external_only, 1},
   {"format", required_argument, 0, 'f'},
@@ -278,6 +281,7 @@ static struct option long_options[] =
   {"size-sort", no_argument, &sort_by_size, 1},
   {"stats", no_argument, &show_stats, 1},
   {"target", required_argument, 0, OPTION_TARGET},
+  {"style", required_argument, NULL, OPTION_STYLE},
   {"defined-only", no_argument, &defined_only, 1},
   {"undefined-only", no_argument, &undefined_only, 1},
   {"version", no_argument, &show_version, 1},
@@ -296,11 +300,13 @@ Usage: %s [-aABCDglnopPrsuvV] [-t radix] [--radix=radix] [--target=bfdname]\n\
        [--debug-syms] [--extern-only] [--print-armap] [--print-file-name]\n\
        [--numeric-sort] [--no-sort] [--reverse-sort] [--size-sort]\n\
        [--undefined-only] [--portability] [-f {bsd,sysv,posix}]\n\
-       [--format={bsd,sysv,posix}] [--demangle] [--no-demangle] [--dynamic]\n\
-       [--defined-only] [--line-numbers]\n\
+       [--format={bsd,sysv,posix}] [--demangle] [--no-demangle]\n\
+       [--demangler=dso:function]\n"), program_name);
+  fprintf (stream, _("\
+       [--style=%s]\n\
+       [--dynamic] [--defined-only] [--line-numbers]\n\
        [--version] [--help]\n\
-       [file...]\n"),
-	   program_name);
+       [file...]\n"), get_demangler_list ());
   list_supported_targets (program_name, stream);
   if (status == 0)
     fprintf (stream, _("Report bugs to %s\n"), REPORT_BUGS_TO);
@@ -373,6 +379,7 @@ main (argc, argv)
 {
   int c;
   int retval;
+  const char *demangler = NULL, *style = NULL;
 
 #if defined (HAVE_SETLOCALE) && defined (HAVE_LC_MESSAGES)
   setlocale (LC_MESSAGES, "");
@@ -452,6 +459,14 @@ main (argc, argv)
 	  target = optarg;
 	  break;
 
+	case OPTION_DEMANGLER:
+	  demangler = optarg;
+	  break;
+
+	case OPTION_STYLE:
+	  style = optarg;
+	  break;
+
 	case 0:		/* A long option that just sets a flag.  */
 	  break;
 
@@ -460,6 +475,8 @@ main (argc, argv)
 	}
     }
 
+  init_demangler (style, NULL, demangler);
+
   if (show_version)
     print_version ("nm");
 
@@ -1036,7 +1053,7 @@ print_symname (format, name, abfd)
       if (bfd_get_symbol_leading_char (abfd) == name[0])
 	name++;
 
-      res = cplus_demangle (name, DMGL_ANSI | DMGL_PARAMS);
+      res = demangle_symbol (name);
       if (res)
 	{
 	  printf (format, res);
