@@ -233,7 +233,7 @@ usage (stream, status)
   -S  --source             Intermix source code with disassembly\n\
   -s  --full-contents      Display the full contents of all sections requested\n\
   -g  --debugging          Display debug information in object file\n\
-  -G  --stabs              Display the STABS contents of an ELF format file\n\
+  -G  --stabs              Display (in raw form) any STABS info in the file\n\
   -t  --syms               Display the contents of the symbol table(s)\n\
   -T  --dynamic-syms       Display the contents of the dynamic symbol table\n\
   -r  --reloc              Display the relocation entries in the file\n\
@@ -263,9 +263,9 @@ usage (stream, status)
       --[no-]show-raw-insn       Display hex alongside symbolic disassembly\n\
       --adjust-vma <offset>      Add <offset> to all displayed section addresses\n\
       --demangler <dso:function> Set dso and demangler function\n\
-      --style {gnu,lucid,arm,hp,edg,gnat,compaq}\n\
+      --style %s \n\
                                  Set demangling style\n\
-\n"));
+\n"), get_demangler_list ());
       list_supported_targets (program_name, stream);
       
       disassembler_usage (stream);
@@ -2660,6 +2660,7 @@ display_target_list ()
 	{
 	  if (bfd_get_error () != bfd_error_invalid_operation)
 	    nonfatal (p->name);
+	  bfd_close_all_done (abfd);
 	  continue;
 	}
 
@@ -2667,6 +2668,7 @@ display_target_list ()
 	if (bfd_set_arch_mach (abfd, (enum bfd_architecture) a, 0))
 	  printf ("  %s\n",
 		  bfd_printable_arch_mach ((enum bfd_architecture) a, 0));
+      bfd_close_all_done (abfd);
     }
   unlink (dummy_name);
   free (dummy_name);
@@ -2734,6 +2736,8 @@ display_info_table (first, last)
 		  putchar ('-');
 		putchar (' ');
 	      }
+	    if (abfd != NULL)
+	      bfd_close_all_done (abfd);
 	  }
 	putchar ('\n');
       }
