@@ -1,5 +1,5 @@
 /* coff object file format
-   Copyright (C) 1989, 90, 91, 92, 94, 95, 96, 97, 98, 1999
+   Copyright (C) 1989, 90, 91, 92, 94, 95, 96, 97, 98, 99, 2000
    Free Software Foundation, Inc.
 
    This file is part of GAS.
@@ -110,12 +110,29 @@
 #endif
 
 #ifdef TC_SH
+
+#ifdef TE_PE
+#define COFF_WITH_PE
+#endif
+
 #include "coff/sh.h"
+
+#ifdef TE_PE
+#define TARGET_FORMAT "pe-shl"
+#else
 #define TARGET_FORMAT					\
   (shl							\
    ? (sh_small ? "coff-shl-small" : "coff-shl")		\
    : (sh_small ? "coff-sh-small" : "coff-sh"))
 #endif
+#endif
+
+#ifdef TC_MIPS
+#define COFF_WITH_PE
+#include "coff/mipspe.h"
+#undef  TARGET_FORMAT
+#define TARGET_FORMAT "pe-mips"
+#endif
 
 #ifdef TC_M88K
 #include "coff/m88k.h"
@@ -132,6 +149,11 @@
 #define TARGET_FORMAT "coff-tic30"
 #endif
 
+#ifdef TC_TIC54X
+#include "coff/tic54x.h"
+#define TARGET_FORMAT "coff1-c54x"
+#endif
+
 #ifdef TC_TIC80
 #include "coff/tic80.h"
 #define TARGET_FORMAT "coff-tic80"
@@ -352,7 +374,7 @@ extern int coff_n_line_nos;
 extern void coff_add_linesym PARAMS ((symbolS *));
 
 
-void c_dot_file_symbol PARAMS ((char *filename));
+void c_dot_file_symbol PARAMS ((const char *filename));
 #define obj_app_file c_dot_file_symbol
 
 extern void coff_frob_symbol PARAMS ((symbolS *, int *));
@@ -371,6 +393,7 @@ extern symbolS *coff_last_function;
 
 /* Forward the segment of a forwarded symbol, handle assignments that
    just copy symbol values, etc.  */
+#ifndef OBJ_COPY_SYMBOL_ATTRIBUTES
 #ifndef TE_I386AIX
 #define OBJ_COPY_SYMBOL_ATTRIBUTES(dest,src) \
   (SF_GET_GET_SEGMENT (dest) \
@@ -382,6 +405,7 @@ extern symbolS *coff_last_function;
    ? (S_SET_SEGMENT (dest, S_GET_SEGMENT (src)), 0) \
    : 0)
 #endif
+#endif
 
 /* sanity check */
 
@@ -821,6 +845,12 @@ extern void obj_coff_pe_handle_link_once ();
 
 #endif /* not BFD_ASSEMBLER */
 
+extern const pseudo_typeS coff_pseudo_table[];
+
+#ifndef obj_pop_insert
+#define obj_pop_insert() pop_insert (coff_pseudo_table)
+#endif
+
 /* In COFF, if a symbol is defined using .def/.val SYM/.endef, it's OK
    to redefine the symbol later on.  This can happen if C symbols use
    a prefix, and a symbol is defined both with and without the prefix,
