@@ -1,5 +1,5 @@
 /* tc-h8500.c -- Assemble code for the Hitachi H8/500
-   Copyright (C) 1993 Free Software Foundation.
+   Copyright (C) 1993, 94, 95, 98, 2000 Free Software Foundation.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -14,8 +14,9 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GAS; see the file COPYING.  If not, write to
-   the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with GAS; see the file COPYING.  If not, write to the Free
+   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
 /*
   Written By Steve Chamberlain
@@ -93,7 +94,7 @@ static struct hash_control *opcode_hash_control;	/* Opcode mnemonics */
 
 /*
   This function is called once, at assembler startup time.  This should
-  set up all the tables, etc that the MD part of the assembler needs
+  set up all the tables, etc. that the MD part of the assembler needs
   */
 
 void
@@ -293,7 +294,7 @@ parse_exp (s, op, page)
 
   expression (op);
   if (op->X_op == O_absent)
-    as_bad ("missing operand");
+    as_bad (_("missing operand"));
   new = input_line_pointer;
   input_line_pointer = save;
   return new;
@@ -333,14 +334,14 @@ skip_colonthing (sign, ptr, exp, def, size8, size16, size24)
 	{
 	  if (!size24)
 	    {
-	      as_bad (":24 not valid for this opcode");
+	      as_bad (_(":24 not valid for this opcode"));
 	    }
 	  ptr += 2;
 	  exp->type = size24;
 	}
       else
 	{
-	  as_bad ("expect :8,:16 or :24");
+	  as_bad (_("expect :8,:16 or :24"));
 	  exp->type = size16;
 	}
     }
@@ -397,7 +398,7 @@ parse_reglist (src, op)
 	}
       else
 	{
-	  as_bad ("syntax error in reg list");
+	  as_bad (_("syntax error in reg list"));
 	  return 0;
 	}
       if (src[idx] == '-')
@@ -415,7 +416,7 @@ parse_reglist (src, op)
 	    }
 	  else
 	    {
-	      as_bad ("missing final register in range");
+	      as_bad (_("missing final register in range"));
 	    }
 	}
       if (src[idx] == ',')
@@ -502,20 +503,20 @@ get_operand (ptr, op, ispage)
 
 	  if (*src != ',')
 	    {
-	      as_bad ("expected @(exp, Rn)");
+	      as_bad (_("expected @(exp, Rn)"));
 	      return;
 	    }
 	  src++;
 	  len = parse_reg (src, &mode, &op->reg);
 	  if (len == 0 || mode != RN)
 	    {
-	      as_bad ("expected @(exp, Rn)");
+	      as_bad (_("expected @(exp, Rn)"));
 	      return;
 	    }
 	  src += len;
 	  if (*src != ')')
 	    {
-	      as_bad ("expected @(exp, Rn)");
+	      as_bad (_("expected @(exp, Rn)"));
 	      return;
 	    }
 	  *ptr = src + 1;
@@ -531,7 +532,7 @@ get_operand (ptr, op, ispage)
 	      src++;
 	      if (mode != RN)
 		{
-		  as_bad ("@Rn+ needs word register");
+		  as_bad (_("@Rn+ needs word register"));
 		  return;
 		}
 	      op->type = RNINC;
@@ -541,7 +542,7 @@ get_operand (ptr, op, ispage)
 	    }
 	  if (mode != RN)
 	    {
-	      as_bad ("@Rn needs word register");
+	      as_bad (_("@Rn needs word register"));
 	      return;
 	    }
 	  op->type = RNIND;
@@ -841,7 +842,7 @@ get_specific (opcode, operands)
 		}
 	      break;
 	    default:
-	      printf ("unhandled %d\n", this_try->arg_type[i]);
+	      printf (_("unhandled %d\n"), this_try->arg_type[i]);
 	      break;
 	    }
 
@@ -869,7 +870,7 @@ check (operand, low, high)
       || operand->X_add_number < low
       || operand->X_add_number > high)
     {
-      as_bad ("operand must be absolute in range %d..%d", low, high);
+      as_bad (_("operand must be absolute in range %d..%d"), low, high);
     }
   return operand->X_add_number;
 }
@@ -932,7 +933,7 @@ build_relaxable_instruction (opcode, operand)
     }
 }
 
-/* Now we know what sort of opcodes it is, lets build the bytes -
+/* Now we know what sort of opcodes it is, let's build the bytes -
  */
 static void
 build_bytes (opcode, operand)
@@ -959,7 +960,7 @@ build_bytes (opcode, operand)
 	  switch (opcode->bytes[index].insert)
 	    {
 	    default:
-	      printf ("failed for %d\n", opcode->bytes[index].insert);
+	      printf (_("failed for %d\n"), opcode->bytes[index].insert);
 	      break;
 	    case 0:
 	      break;
@@ -1080,13 +1081,13 @@ build_bytes (opcode, operand)
 }
 
 /* This is the guts of the machine-dependent assembler.  STR points to a
-   machine dependent instruction.  This funciton is supposed to emit
+   machine dependent instruction.  This function is supposed to emit
    the frags/bytes it assembles to.
    */
 
 void
-DEFUN (md_assemble, (str),
-       char *str)
+md_assemble (str)
+     char *str;
 {
   char *op_start;
   char *op_end;
@@ -1103,8 +1104,7 @@ DEFUN (md_assemble, (str),
 
   /* find the op code end */
   for (op_start = op_end = str;
-       *op_end &&
-       !is_end_of_line[*op_end] && *op_end != ' ';
+       !is_end_of_line[(unsigned char) *op_end] && *op_end != ' ';
        op_end++)
     {
       if (			/**op_end != '.'
@@ -1118,14 +1118,14 @@ DEFUN (md_assemble, (str),
 
   if (op_end == op_start)
     {
-      as_bad ("can't find opcode ");
+      as_bad (_("can't find opcode "));
     }
 
   opcode = (h8500_opcode_info *) hash_find (opcode_hash_control, name);
 
   if (opcode == NULL)
     {
-      as_bad ("unknown opcode");
+      as_bad (_("unknown opcode"));
       return;
     }
 
@@ -1141,7 +1141,7 @@ DEFUN (md_assemble, (str),
 
       where[0] = 0x0;
       where[1] = 0x0;
-      as_bad ("invalid operands for opcode");
+      as_bad (_("invalid operands for opcode"));
       return;
     }
 
@@ -1150,24 +1150,24 @@ DEFUN (md_assemble, (str),
 }
 
 void
-DEFUN (tc_crawl_symbol_chain, (headers),
-       object_headers * headers)
+tc_crawl_symbol_chain (headers)
+     object_headers *headers;
 {
-  printf ("call to tc_crawl_symbol_chain \n");
+  printf (_("call to tc_crawl_symbol_chain \n"));
 }
 
 symbolS *
-DEFUN (md_undefined_symbol, (name),
-       char *name)
+md_undefined_symbol (name)
+     char *name;
 {
   return 0;
 }
 
 void
-DEFUN (tc_headers_hook, (headers),
-       object_headers * headers)
+tc_headers_hook (headers)
+     object_headers *headers;
 {
-  printf ("call to tc_headers_hook \n");
+  printf (_("call to tc_headers_hook \n"));
 }
 
 /* Various routines to kill one day */
@@ -1175,8 +1175,8 @@ DEFUN (tc_headers_hook, (headers),
 #define MAX_LITTLENUMS 6
 
 /* Turn a string in input_line_pointer into a floating point constant of type
-   type, and store the appropriate bytes in *litP.  The number of LITTLENUMS
-   emitted is stored in *sizeP .  An error message is returned, or NULL on OK.
+   type, and store the appropriate bytes in *LITP.  The number of LITTLENUMS
+   emitted is stored in *SIZEP .  An error message is returned, or NULL on OK.
    */
 char *
 md_atof (type, litP, sizeP)
@@ -1218,7 +1218,7 @@ md_atof (type, litP, sizeP)
 
     default:
       *sizeP = 0;
-      return "Bad call to MD_ATOF()";
+      return _("Bad call to MD_ATOF()");
     }
   t = atof_ieee (input_line_pointer, type, words);
   if (t)
@@ -1253,36 +1253,13 @@ md_show_usage (stream)
 {
 }
 
-int md_short_jump_size;
-
 void
 tc_aout_fix_to_chars ()
 {
-  printf ("call to tc_aout_fix_to_chars \n");
+  printf (_("call to tc_aout_fix_to_chars \n"));
   abort ();
 }
 
-void
-md_create_short_jump (ptr, from_addr, to_addr, frag, to_symbol)
-     char *ptr;
-     addressT from_addr;
-     addressT to_addr;
-     fragS *frag;
-     symbolS *to_symbol;
-{
-  as_fatal ("failed sanity check.");
-}
-
-void
-md_create_long_jump (ptr, from_addr, to_addr, frag, to_symbol)
-     char *ptr;
-     addressT from_addr, to_addr;
-     fragS *frag;
-     symbolS *to_symbol;
-{
-  as_fatal ("failed sanity check.");
-}
-
 static
 void
 wordify_scb (buffer, disp_size, inst_size)
@@ -1480,8 +1457,6 @@ md_apply_fix (fixP, val)
     }
 }
 
-int md_long_jump_size;
-
 /*
 called just before address relaxation, return the length
 by which a fragment must grow to reach it's destination
