@@ -74,14 +74,14 @@ const char line_comment_chars[] = "#/";
 
 const int md_reloc_size = 8;
 
-static int do_jsri2bsr = 0;	/* change here from 1 by Cruess 19 August 97 */
+static int do_jsri2bsr = 0;	/* Change here from 1 by Cruess 19 August 97.  */
 static int sifilter_mode = 0;
 
 const char EXP_CHARS[] = "eE";
 
-/* Chars that mean this number is a floating point constant */
-/* As in 0f12.456 */
-/* or    0d1.2345e12 */
+/* Chars that mean this number is a floating point constant
+    As in 0f12.456
+    or    0d1.2345e12  */
 const char FLT_CHARS[] = "rRsSfFdDxXpP";
 
 #define C(what,length) (((what) << 2) + (length))
@@ -113,7 +113,7 @@ cpu_type;
 
 cpu_type cpu = M340;
 
-/* Initialize the relax table */
+/* Initialize the relax table.  */
 const relax_typeS md_relax_table[] =
 {
 {    1,     1,	     0, 0 },			/* 0: unused */  
@@ -131,7 +131,7 @@ const relax_typeS md_relax_table[] =
 {    0,     0,	     0, 0 }			/*12: unused */  
 };
 
-/* LITERAL POOL DATA STRUCTURES */
+/* Literal pool data structures.  */
 struct literal
 {
   unsigned short  refcnt;
@@ -158,18 +158,18 @@ static unsigned long poolspan;
    -- so we have to be smaller than 1018 and since we deal with 2-byte
    instructions, the next good choice is 1016.
    -- Note we have a test case that fails when we've got 1018 here.  */
-#define SPANPANIC	(1016)		/* 1024 - 1 entry - 2 byte rounding */
+#define SPANPANIC	(1016)		/* 1024 - 1 entry - 2 byte rounding.  */
 #define SPANCLOSE	(900)
 #define SPANEXIT	(600)
-static symbolS * poolsym;		/* label for current pool */
+static symbolS * poolsym;		/* label for current pool.  */
 static char poolname[8];
-static struct hash_control * opcode_hash_control;	/* Opcode mnemonics */
+static struct hash_control * opcode_hash_control;	/* Opcode mnemonics.  */
 
 /* This table describes all the machine specific pseudo-ops the assembler
    has to support.  The fields are:
      Pseudo-op name without dot
      Function to call to execute this pseudo-op
-     Integer arg to pass to the function   */
+     Integer arg to pass to the function.   */
 const pseudo_typeS md_pseudo_table[] =
 {
   { "export",   s_globl,          0 },
@@ -239,7 +239,7 @@ mcore_cons (nbytes)
       int    commas = 1;
       
       /* Count the number of commas on the line.  */
-      while (! is_end_of_line [* ptr])
+      while (! is_end_of_line [(unsigned char) * ptr])
 	commas += * ptr ++ == ',';
       
       poolspan += nbytes * commas;
@@ -268,7 +268,7 @@ mcore_float_cons (float_type)
 #endif
       
       /* Count the number of commas on the line.  */
-      while (! is_end_of_line [* ptr])
+      while (! is_end_of_line [(unsigned char) * ptr])
 	commas += * ptr ++ == ',';
 
       /* We would like to compute "hex_float (float_type) * commas"
@@ -301,7 +301,7 @@ mcore_stringer (append_zero)
 	 estimate, which is OK, and automatically allows for the
 	 appending a zero byte, since the real string(s) is/are
 	 required to be enclosed in double quotes.  */
-      while (! is_end_of_line [* ptr])
+      while (! is_end_of_line [(unsigned char) * ptr])
 	ptr ++;
 
       poolspan += ptr - input_line_pointer;
@@ -975,7 +975,7 @@ md_assemble (str)
 
   /* Find the op code end.  */
   for (op_start = op_end = str;
-       * op_end && nlen < 20 && !is_end_of_line [*op_end] && *op_end != ' ';
+       nlen < 20 && !is_end_of_line [(unsigned char) *op_end] && *op_end != ' ';
        op_end++)
     {
       name[nlen] = op_start[nlen];
@@ -2146,8 +2146,8 @@ md_apply_fix3 (fixP, valp, segment)
 	}
       else
 	{
-	  buf[0] |= ((val >> 8) & 0x7);
-	  buf[1] |= (val & 0xff);
+	  buf[1] |= ((val >> 8) & 0x7);
+	  buf[0] |= (val & 0xff);
 	}
       break;
 
