@@ -1,5 +1,6 @@
 /* tc-z8k.c -- Assemble code for the Zilog Z800n
-   Copyright (C) 1992 Free Software Foundation.
+   Copyright (C) 1992, 93, 94, 95, 96, 97, 98, 99, 2000
+   Free Software Foundation.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -14,8 +15,9 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GAS; see the file COPYING.  If not, write to
-   the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with GAS; see the file COPYING.  If not, write to the Free
+   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
 /*
   Written By Steve Chamberlain
@@ -30,12 +32,9 @@
 #include "bfd.h"
 #include <ctype.h>
 
-const char comment_chars[] =
-{'!', 0};
-const char line_separator_chars[] =
-{';', 0};
-const char line_comment_chars[] =
-{'#', 0};
+const char comment_chars[] = "!";
+const char line_comment_chars[] = "#";
+const char line_separator_chars[] = ";";
 
 extern int machine;
 extern int coff_flags;
@@ -71,7 +70,7 @@ static
 void
 even ()
 {
-  frag_align (1, 0);
+  frag_align (1, 0, 0);
   record_alignment (now_seg, 1);
 }
 
@@ -222,9 +221,9 @@ int the_flags;
 int the_interrupt;
 
 char *
-DEFUN (whatreg, (reg, src),
-       int *reg AND
-       char *src)
+whatreg (reg, src)
+     int *reg;
+     char *src;
 {
   if (isdigit (src[1]))
     {
@@ -256,10 +255,10 @@ DEFUN (whatreg, (reg, src),
 
 /* try and parse a reg name, returns number of chars consumed */
 char *
-DEFUN (parse_reg, (src, mode, reg),
-       char *src AND
-       int *mode AND
-       unsigned int *reg)
+parse_reg (src, mode, reg)
+     char *src;
+     int *mode;
+     unsigned int *reg;
 {
   char *res = 0;
   char regno;
@@ -286,7 +285,7 @@ DEFUN (parse_reg, (src, mode, reg),
           res = whatreg (reg, src + 2);
 	  regno = *reg;
 	  if (regno > 14)
-          	as_warn ("register rr%d, out of range.",regno);
+          	as_warn (_("register rr%d, out of range."),regno);
         }
       else if (src[1] == 'h')
         {
@@ -294,7 +293,7 @@ DEFUN (parse_reg, (src, mode, reg),
           res = whatreg (reg, src + 2);
 	  regno = *reg;
 	  if (regno > 7)
-          	as_warn ("register rh%d, out of range.",regno);
+          	as_warn (_("register rh%d, out of range."),regno);
         }
       else if (src[1] == 'l')
         {
@@ -302,7 +301,7 @@ DEFUN (parse_reg, (src, mode, reg),
           res = whatreg (reg, src + 2);
 	  regno = *reg;
 	  if (regno > 7)
-          	as_warn ("register rl%d, out of range.",regno);
+          	as_warn (_("register rl%d, out of range."),regno);
           *reg += 8;
         }
       else if (src[1] == 'q')
@@ -311,7 +310,7 @@ DEFUN (parse_reg, (src, mode, reg),
           res = whatreg (reg, src + 2);
 	  regno = *reg;
 	  if (regno > 12)
-          	as_warn ("register rq%d, out of range.",regno);
+          	as_warn (_("register rq%d, out of range."),regno);
         }
       else
         {
@@ -319,7 +318,7 @@ DEFUN (parse_reg, (src, mode, reg),
           res = whatreg (reg, src + 1);
 	  regno = *reg;
 	  if (regno > 15)
-          	as_warn ("register r%d, out of range.",regno);
+          	as_warn (_("register r%d, out of range."),regno);
         }
     }
   return res;
@@ -327,9 +326,9 @@ DEFUN (parse_reg, (src, mode, reg),
 }
 
 char *
-DEFUN (parse_exp, (s, op),
-       char *s AND
-       expressionS * op)
+parse_exp (s, op)
+     char *s;
+     expressionS *op;
 {
   char *save = input_line_pointer;
   char *new;
@@ -337,7 +336,7 @@ DEFUN (parse_exp, (s, op),
   input_line_pointer = s;
   expression (op);
   if (op->X_op == O_absent)
-    as_bad ("missing operand");
+    as_bad (_("missing operand"));
   new = input_line_pointer;
   input_line_pointer = save;
   return new;
@@ -362,46 +361,46 @@ DEFUN (parse_exp, (s, op),
 
 static
 char *
-DEFUN (checkfor, (ptr, what),
-       char *ptr AND
-       char what)
+checkfor (ptr, what)
+     char *ptr;
+     char what;
 {
   if (*ptr == what)
     ptr++;
   else
     {
-      as_bad ("expected %c", what);
+      as_bad (_("expected %c"), what);
     }
   return ptr;
 }
 
 /* Make sure the mode supplied is the size of a word */
 static void
-DEFUN (regword, (mode, string),
-       int mode AND
-       char *string)
+regword (mode, string)
+     int mode;
+     char *string;
 {
   int ok;
 
   ok = CLASS_REG_WORD;
   if (ok != mode)
     {
-      as_bad ("register is wrong size for a word %s", string);
+      as_bad (_("register is wrong size for a word %s"), string);
     }
 }
 
 /* Make sure the mode supplied is the size of an address */
 static void
-DEFUN (regaddr, (mode, string),
-       int mode AND
-       char *string)
+regaddr (mode, string)
+     int mode;
+     char *string;
 {
   int ok;
 
   ok = segmented_mode ? CLASS_REG_LONG : CLASS_REG_WORD;
   if (ok != mode)
     {
-      as_bad ("register is wrong size for address %s", string);
+      as_bad (_("register is wrong size for address %s"), string);
     }
 }
 
@@ -425,10 +424,10 @@ struct ctrl_names ctrl_table[] =
 };
    
 static void
-DEFUN (get_ctrl_operand, (ptr, mode, dst),
-       char **ptr AND
-       struct z8k_op *mode AND
-       unsigned int dst)
+get_ctrl_operand (ptr, mode, dst)
+     char **ptr;
+     struct z8k_op *mode;
+     unsigned int dst;
 {
   char *src = *ptr;
   int r;
@@ -475,10 +474,10 @@ struct flag_names flag_table[] =
 };
 
 static void
-DEFUN (get_flags_operand, (ptr, mode, dst),
-       char **ptr AND
-       struct z8k_op *mode AND
-       unsigned int dst)
+get_flags_operand (ptr, mode, dst)
+     char **ptr;
+     struct z8k_op *mode;
+     unsigned int dst;
 {
   char *src = *ptr;
   int r;
@@ -529,10 +528,10 @@ struct interrupt_names intr_table[] =
 };
 
 static void
-DEFUN (get_interrupt_operand, (ptr, mode, dst),
-       char **ptr AND
-       struct z8k_op *mode AND
-       unsigned int dst)
+get_interrupt_operand (ptr, mode, dst)
+     char **ptr;
+     struct z8k_op *mode;
+     unsigned int dst;
 {
   char *src = *ptr;
   int r;
@@ -595,10 +594,10 @@ struct cc_names table[] =
 };
 
 static void
-DEFUN (get_cc_operand, (ptr, mode, dst),
-       char **ptr AND
-       struct z8k_op *mode AND
-       unsigned int dst)
+get_cc_operand (ptr, mode, dst)
+     char **ptr;
+     struct z8k_op *mode;
+     unsigned int dst;
 {
   char *src = *ptr;
   int r;
@@ -676,7 +675,7 @@ get_operand (ptr, mode, dst)
 
 		  if (*src != ')')
 		    {
-		      as_bad ("Missing ) in ra(rb)");
+		      as_bad (_("Missing ) in ra(rb)"));
 		    }
 		  else
 		    {
@@ -845,9 +844,9 @@ char *savptr;
 
 static
 opcode_entry_type *
-DEFUN (get_specific, (opcode, operands),
-       opcode_entry_type * opcode AND
-       op_type * operands)
+get_specific (opcode, operands)
+     opcode_entry_type *opcode;
+     op_type *operands;
 
 {
   opcode_entry_type *this_try = opcode;
@@ -928,10 +927,10 @@ DEFUN (get_specific, (opcode, operands),
 }
 
 static void
-DEFUN (check_operand, (operand, width, string),
-       struct z8k_op *operand AND
-       unsigned int width AND
-       char *string)
+check_operand (operand, width, string)
+     struct z8k_op *operand;
+     unsigned int width;
+     char *string;
 {
   if (operand->exp.X_add_symbol == 0
       && operand->exp.X_op_symbol == 0)
@@ -943,7 +942,7 @@ DEFUN (check_operand, (operand, width, string),
       if ((operand->exp.X_add_number & ~width) != 0 &&
 	  (operand->exp.X_add_number | width) != (~0))
 	{
-	  as_warn ("operand %s0x%x out of range.", string, operand->exp.X_add_number);
+	  as_warn (_("operand %s0x%x out of range."), string, operand->exp.X_add_number);
 	}
     }
 
@@ -952,10 +951,10 @@ DEFUN (check_operand, (operand, width, string),
 static char buffer[20];
 
 static void
-DEFUN (newfix, (ptr, type, operand),
-       int ptr AND
-       int type AND
-       expressionS * operand)
+newfix (ptr, type, operand)
+     int ptr;
+     int type;
+     expressionS *operand;
 {
   if (operand->X_add_symbol
       || operand->X_op_symbol
@@ -971,11 +970,11 @@ DEFUN (newfix, (ptr, type, operand),
 }
 
 static char *
-DEFUN (apply_fix, (ptr, type, operand, size),
-       char *ptr AND
-       int type AND
-       expressionS * operand AND
-       int size)
+apply_fix (ptr, type, operand, size)
+     char *ptr;
+     int type;
+     expressionS *operand;
+     int size;
 {
   int n = operand->X_add_number;
 
@@ -1082,12 +1081,12 @@ top:;
 		}
 	      else if (imm_operand->X_add_number != 1)
 		{
-		  as_bad ("immediate must be 1 or 2");
+		  as_bad (_("immediate must be 1 or 2"));
 		}
 	    }
 	  else
 	    {
-	      as_bad ("immediate 1 or 2 expected");
+	      as_bad (_("immediate 1 or 2 expected"));
 	    }
 	  output_ptr++;
 	  break;
@@ -1115,7 +1114,7 @@ top:;
 	case CLASS_REGN0:
 	  if (reg[c & 0xf] == 0)
 	    {
-	      as_bad ("can't use R0 here");
+	      as_bad (_("can't use R0 here"));
 	    }
 	case CLASS_REG:
 	case CLASS_REG_BYTE:
@@ -1186,13 +1185,12 @@ top:;
 }
 
 /* This is the guts of the machine-dependent assembler.  STR points to a
-   machine dependent instruction.  This funciton is supposed to emit
-   the frags/bytes it assembles to.
-   */
+   machine dependent instruction.  This function is supposed to emit
+   the frags/bytes it assembles to.  */
 
 void
-DEFUN (md_assemble, (str),
-       char *str)
+md_assemble (str)
+     char *str;
 {
   char *op_start;
   char *op_end;
@@ -1219,7 +1217,7 @@ DEFUN (md_assemble, (str),
 
   if (op_end == op_start)
     {
-      as_bad ("can't find opcode ");
+      as_bad (_("can't find opcode "));
     }
   c = *op_end;
 
@@ -1231,7 +1229,7 @@ DEFUN (md_assemble, (str),
 
   if (opcode == NULL)
     {
-      as_bad ("unknown opcode");
+      as_bad (_("unknown opcode"));
       return;
     }
 
@@ -1274,7 +1272,7 @@ DEFUN (md_assemble, (str),
 	  where[0] = 0x0;
 	  where[1] = 0x0;
 
-	  as_bad ("Can't find opcode to match operands");
+	  as_bad (_("Can't find opcode to match operands"));
 	  return;
 	}
 
@@ -1283,24 +1281,24 @@ DEFUN (md_assemble, (str),
 }
 
 void
-DEFUN (tc_crawl_symbol_chain, (headers),
-       object_headers * headers)
+tc_crawl_symbol_chain (headers)
+     object_headers *headers;
 {
-  printf ("call to tc_crawl_symbol_chain \n");
+  printf (_("call to tc_crawl_symbol_chain \n"));
 }
 
 symbolS *
-DEFUN (md_undefined_symbol, (name),
-       char *name)
+md_undefined_symbol (name)
+     char *name;
 {
   return 0;
 }
 
 void
-DEFUN (tc_headers_hook, (headers),
-       object_headers * headers)
+tc_headers_hook (headers)
+     object_headers *headers;
 {
-  printf ("call to tc_headers_hook \n");
+  printf (_("call to tc_headers_hook \n"));
 }
 
 /* Various routines to kill one day */
@@ -1351,7 +1349,7 @@ md_atof (type, litP, sizeP)
 
     default:
       *sizeP = 0;
-      return "Bad call to MD_ATOF()";
+      return _("Bad call to MD_ATOF()");
     }
   t = atof_ieee (input_line_pointer, type, words);
   if (t)
@@ -1386,7 +1384,7 @@ md_parse_option (c, arg)
 	s_unseg ();
       else
 	{
-	  as_bad ("invalid architecture -z%s", arg);
+	  as_bad (_("invalid architecture -z%s"), arg);
 	  return 0;
 	}
       break;
@@ -1402,56 +1400,33 @@ void
 md_show_usage (stream)
      FILE *stream;
 {
-  fprintf(stream, "\
+  fprintf(stream, _("\
 Z8K options:\n\
 -z8001			generate segmented code\n\
--z8002			generate unsegmented code\n");
+-z8002			generate unsegmented code\n"));
 }
 
-int md_short_jump_size;
-
 void
 tc_aout_fix_to_chars ()
 {
-  printf ("call to tc_aout_fix_to_chars \n");
+  printf (_("call to tc_aout_fix_to_chars \n"));
   abort ();
 }
 
-void
-md_create_short_jump (ptr, from_addr, to_addr, frag, to_symbol)
-     char *ptr;
-     addressT from_addr;
-     addressT to_addr;
-     fragS *frag;
-     symbolS *to_symbol;
-{
-  as_fatal ("failed sanity check.");
-}
-
-void
-md_create_long_jump (ptr, from_addr, to_addr, frag, to_symbol)
-     char *ptr;
-     addressT from_addr, to_addr;
-     fragS *frag;
-     symbolS *to_symbol;
-{
-  as_fatal ("failed sanity check.");
-}
-
 void
 md_convert_frag (headers, seg, fragP)
      object_headers *headers;
      segT seg;
      fragS *fragP;
 {
-  printf ("call to md_convert_frag \n");
+  printf (_("call to md_convert_frag \n"));
   abort ();
 }
 
 valueT
-DEFUN (md_section_align, (seg, size),
-       segT seg AND
-       valueT size)
+md_section_align (seg, size)
+     segT seg;
+     valueT size;
 {
   return ((size + (1 << section_alignment[(int) seg]) - 1) & (-1 << section_alignment[(int) seg]));
 
@@ -1514,23 +1489,22 @@ md_apply_fix (fixP, val)
     }
 }
 
-int md_long_jump_size;
 int
 md_estimate_size_before_relax (fragP, segment_type)
      register fragS *fragP;
      register segT segment_type;
 {
-  printf ("call tomd_estimate_size_before_relax \n");
+  printf (_("call tomd_estimate_size_before_relax \n"));
   abort ();
 }
 
 /* Put number into target byte order */
 
 void
-DEFUN (md_number_to_chars, (ptr, use, nbytes),
-       char *ptr AND
-       valueT use AND
-       int nbytes)
+md_number_to_chars (ptr, use, nbytes)
+     char *ptr;
+     valueT use;
+     int nbytes;
 {
   number_to_chars_bigendian (ptr, use, nbytes);
 }
@@ -1543,7 +1517,7 @@ md_pcrel_from (fixP)
 
 void
 tc_coff_symbol_emit_hook (s)
-     struct symbol *s;
+     symbolS *s;
 {
 }
 
@@ -1563,7 +1537,7 @@ tc_reloc_mangle (fix_ptr, intr, base)
       symbolS *sub = fix_ptr->fx_subsy;
       if (S_GET_SEGMENT(add) != S_GET_SEGMENT(sub))
 	{
-	  as_bad("Can't subtract symbols in different sections %s %s",
+	  as_bad(_("Can't subtract symbols in different sections %s %s"),
 		 S_GET_NAME(add), S_GET_NAME(sub));
 	}
       else {
