@@ -15,12 +15,11 @@
    
    You should have received a copy of the GNU General Public License
    along with GAS; see the file COPYING.  If not, write to
-   the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
-/*
-	HP PA-RISC and OSF/1 support was contributed by the Center for
-	Software Science at the University of Utah.
- */
+/* HP PA-RISC and OSF/1 support was contributed by the Center for
+	Software Science at the University of Utah.  */
 
-/* these define interfaces */
+/* These define interfaces.  */
 #include "obj-format.h"
