@@ -541,8 +541,8 @@ integer_constant (radix, expressionP)
 	}
     }
 
-  if ((NUMBERS_WITH_SUFFIX || flag_m68k_mri) 
-      && suffix != NULL 
+  if ((NUMBERS_WITH_SUFFIX || flag_m68k_mri)
+      && suffix != NULL
       && input_line_pointer - 1 == suffix)
     c = *input_line_pointer++;
 
@@ -799,6 +799,9 @@ operand (expressionP)
   SKIP_WHITESPACE ();		/* leading whitespace is part of operand. */
   c = *input_line_pointer++;	/* input_line_pointer->past char in c. */
 
+  if (is_end_of_line[(unsigned char) c])
+    goto eol;
+
   switch (c)
     {
     case '1':
@@ -812,11 +815,17 @@ operand (expressionP)
     case '9':
       input_line_pointer--;
 
-      integer_constant ((NUMBERS_WITH_SUFFIX || flag_m68k_mri) 
+      integer_constant ((NUMBERS_WITH_SUFFIX || flag_m68k_mri)
                         ? 0 : 10,
                         expressionP);
       break;
 
+#ifdef LITERAL_PREFIXDOLLAR_HEX
+    case '$':
+      integer_constant (16, expressionP);
+      break;
+#endif
+
     case '0':
       /* non-decimal radix */
 
@@ -1001,12 +1010,16 @@ operand (expressionP)
       /* didn't begin with digit & not a name */
       segment = expression (expressionP);
       /* Expression() will pass trailing whitespace */
-      if ((c == '(' && *input_line_pointer++ != ')')
-	  || (c == '[' && *input_line_pointer++ != ']'))
+      if ((c == '(' && *input_line_pointer != ')')
+	  || (c == '[' && *input_line_pointer != ']'))
 	{
-	  as_bad (_("Missing ')' assumed"));
-	  input_line_pointer--;
+#ifdef RELAX_PAREN_GROUPING
+	  if (c != '(')
+#endif
+	    as_bad (_("Missing '%c' assumed"), c == '(' ? ')' : ']');
 	}
+      else
+        input_line_pointer++;
       SKIP_WHITESPACE ();
       /* here with input_line_pointer->char after "(...)" */
       return segment;
@@ -1166,9 +1179,8 @@ operand (expressionP)
 	{
 	  goto isname;
 	}
+
     case ',':
-    case '\n':
-    case '\0':
     eol:
       /* can't imagine any other kind of operand */
       expressionP->X_op = O_absent;
@@ -1211,8 +1223,6 @@ operand (expressionP)
 #ifdef TC_M68K
     de_fault:
 #endif
-      if (is_end_of_line[(unsigned char) c])
-	goto eol;
       if (is_name_beginner (c))	/* here if did not begin with a digit */
 	{
 	  /*
@@ -1571,6 +1581,9 @@ operator ()
 
   c = *input_line_pointer & 0xff;
 
+  if (is_end_of_line[c])
+    return O_illegal;
+
   switch (c)
     {
     default:
