@@ -29,6 +29,7 @@
 #include "hist.h"
 #include "source.h"
 #include "sym_ids.h"
+#include "demangle.h"
 
 const char *whoami;
 const char *function_mapping_file;
@@ -79,6 +80,8 @@ static char *default_excluded_list[] =
 
 #define OPTION_DEMANGLE		(150)
 #define OPTION_NO_DEMANGLE	(OPTION_DEMANGLE + 1)
+#define OPTION_DEMANGLER	(OPTION_NO_DEMANGLE + 1)
+#define OPTION_STYLE		(OPTION_DEMANGLER + 1)
 
 static struct option long_options[] =
 {
@@ -106,12 +109,14 @@ static struct option long_options[] =
   {"all-lines", no_argument, 0, 'x'},
   {"demangle", no_argument, 0, OPTION_DEMANGLE},
   {"no-demangle", no_argument, 0, OPTION_NO_DEMANGLE},
+  {"demangler", required_argument, NULL, OPTION_DEMANGLER},
   {"directory-path", required_argument, 0, 'I'},
   {"display-unused-functions", no_argument, 0, 'z'},
   {"min-count", required_argument, 0, 'm'},
   {"print-path", no_argument, 0, 'L'},
   {"separate-files", no_argument, 0, 'y'},
   {"static-call-graph", no_argument, 0, 'c'},
+  {"style", required_argument, NULL, OPTION_STYLE},
   {"table-length", required_argument, 0, 't'},
   {"time", required_argument, 0, 'n'},
   {"no-time", required_argument, 0, 'N'},
@@ -137,7 +142,6 @@ static struct option long_options[] =
   {0, no_argument, 0, 0}
 };
 
-
 static void
 DEFUN (usage, (stream, status), FILE * stream AND int status)
 {
@@ -154,8 +158,11 @@ Usage: %s [-[abcDhilLsTvwxyz]] [-[ACeEfFJnNOpPqQZ][name]] [-I dirs]\n\
 	[--static-call-graph] [--sum] [--table-length=len] [--traditional]\n\
 	[--version] [--width=n] [--ignore-non-functions]\n\
 	[--demangle] [--no-demangle]\n\
+	[--demangler=dso:function]\n"), whoami);
+  fprintf (stream, _("\
+	[--style=%s]\n\
 	[image-file] [profile-file...]\n"),
-	   whoami);
+	   get_demangler_list ());
   if (status == 0)
     fprintf (stream, _("Report bugs to %s\n"), REPORT_BUGS_TO);
   done (status);
@@ -168,6 +175,7 @@ DEFUN (main, (argc, argv), int argc AND char **argv)
   char **sp, *str;
   Sym **cg = 0;
   int ch, user_specified = 0;
+  const char *demangler = NULL, *style = NULL;
 
 #if defined (HAVE_SETLOCALE) && defined (HAVE_LC_MESSAGES)
   setlocale (LC_MESSAGES, "");
@@ -179,7 +187,7 @@ DEFUN (main, (argc, argv), int argc AND char **argv)
   xmalloc_set_program_name (whoami);
 
   while ((ch = getopt_long (argc, argv,
-	"aA::bBcCdD::e:E:f:F:hiI:J::k:lLm:n::N::O:p::P::q::Q::st:Tvw:xyzZ::",
+	"aA::bBcCd::De:E:f:F:hiI:J::k:lLm:n::N::O:p::P::q::Q::st:Tvw:xyzZ::",
 			    long_options, 0))
 	 != EOF)
     {
@@ -429,11 +437,19 @@ This program is free software.  This program has absolutely no warranty.\n"));
 	case OPTION_NO_DEMANGLE:
 	  demangle = FALSE;
 	  break;
+	case OPTION_DEMANGLER:
+	  demangler = optarg;
+	  break;
+	case OPTION_STYLE:
+	  style = optarg;
+	  break;
 	default:
 	  usage (stderr, 1);
 	}
     }
 
+  init_demangler (style, NULL, demangler);
+
   /* Don't allow both ordering options, they modify the arc data in-place.  */
   if ((user_specified & STYLE_FUNCTION_ORDER)
       && (user_specified & STYLE_FILE_ORDER))
