@@ -126,7 +126,8 @@ int parsing_defsym = 0;
 #define OPTION_NO_UNDEFINED		(OPTION_MPC860C0 + 1)
 #define OPTION_INIT                     (OPTION_NO_UNDEFINED + 1)
 #define OPTION_FINI                     (OPTION_INIT + 1)
-#define OPTION_MULTILIB_DIR		(OPTION_FINI + 1)
+#define OPTION_SECTION_START		(OPTION_FINI + 1)
+#define OPTION_MULTILIB_DIR		(OPTION_SECTION_START + 1)
 
 /* The long options.  This structure is used for both the option
    parsing and the help text.  */
@@ -207,6 +208,8 @@ static const struct ld_option ld_options[] =
       'O', NULL, N_("Optimize output file"), ONE_DASH },
   { {"Qy", no_argument, NULL, OPTION_IGNORE},
       '\0', NULL, N_("Ignored for SVR4 compatibility"), ONE_DASH },
+  { {"emit-relocs", no_argument, NULL, 'q'},
+      'q', NULL, "Generate relocations in final output", TWO_DASHES },
   { {"relocateable", no_argument, NULL, 'r'},
       'r', NULL, N_("Generate relocateable output"), TWO_DASHES },
   { {NULL, no_argument, NULL, '\0'},
@@ -339,11 +342,13 @@ static const struct ld_option ld_options[] =
   { {"stats", no_argument, NULL, OPTION_STATS},
       '\0', NULL, N_("Print memory usage statistics"), TWO_DASHES },
   { {"style", required_argument, NULL, OPTION_STYLE},
-      '\0', "{gnu,lucid,arm,hp,edg,gnat,compaq}", N_("Set demangling style"), TWO_DASHES },
+      '\0', demangler_list_buffer, N_("Set demangling style"), TWO_DASHES },
   { {"task-link", required_argument, NULL, OPTION_TASK_LINK},
       '\0', N_("SYMBOL"), N_("Do task level linking"), TWO_DASHES },
   { {"traditional-format", no_argument, NULL, OPTION_TRADITIONAL_FORMAT},
       '\0', NULL, N_("Use same format as native linker"), TWO_DASHES },
+  { {"section-start", required_argument, NULL, OPTION_SECTION_START},
+      '\0', N_("SECTION=ADDRESS"), N_("Set address of named section"), TWO_DASHES },
   { {"Tbss", required_argument, NULL, OPTION_TBSS},
       '\0', N_("ADDRESS"), N_("Set address of .bss section"), ONE_DASH },
   { {"Tdata", required_argument, NULL, OPTION_TDATA},
@@ -665,6 +670,7 @@ parse_args (argc, argv)
 	  command_line.gc_sections = true;
 	  break;
 	case OPTION_HELP:
+	  init_demangler (style, NULL, demangler);
 	  help ();
 	  xexit (0);
 	  break;
@@ -730,6 +736,9 @@ parse_args (argc, argv)
 	case OPTION_OFORMAT:
 	  lang_add_output_format (optarg, (char *) NULL, (char *) NULL, 0);
 	  break;
+	case 'q':
+	  link_info.emitrelocations = true;
+	  break;
 	case 'i':
 	case 'r':
 	  link_info.relocateable = true;
@@ -856,6 +865,39 @@ parse_args (argc, argv)
 	case OPTION_MULTILIB_DIR:
 	  command_line.multilib_dir = buystring (optarg);
 	  break;
+	case OPTION_SECTION_START:
+	  {
+	    char *optarg2;
+
+	    /* Check for <something>=<somthing>...  */
+	    optarg2 = strchr (optarg, '=');
+	    if (optarg2 == NULL)
+	      {
+		fprintf (stderr,
+			 _("%s: Invalid argument to option \"--section-start\"\n"),
+			 program_name);
+		xexit (1);
+	      }
+
+	    optarg2 ++;
+	    
+	    /* So far so good.  Are all the args present?  */
+	    if ((*optarg == '\0') || (*optarg2 == '\0'))
+	      {
+		fprintf (stderr,
+			 _("%s: Missing argument(s) to option \"--section-start\"\n"),
+			 program_name);
+		xexit (1);
+	      }
+
+	    optarg2[-1] = '\0';
+
+	    /* Then set it...  */
+	    set_section_start (optarg, optarg2);
+	    
+	    optarg2[-1] = '=';
+	  }
+	  break;
 	case OPTION_TBSS:
 	  set_section_start (".bss", optarg);
 	  break;
@@ -897,7 +939,7 @@ parse_args (argc, argv)
 	case OPTION_VERSION:
 	  /* This output is intended to follow the GNU standards document.  */
 	  printf ("GNU ld %s\n", ld_program_version);
-	  printf (_("Copyright 1997 Free Software Foundation, Inc.\n"));
+	  printf (_("Copyright 2000 Free Software Foundation, Inc.\n"));
 	  printf (_("\
 This program is free software; you may redistribute it under the terms of\n\
 the GNU General Public License.  This program has absolutely no warranty.\n"));
